//---------------------------------------------------------------------------

#ifndef DirectSoundScopeH
#define DirectSoundScopeH
//---------------------------------------------------------------------------

#include "..\miniscope\device\MiniscopeDeviceCapabilities.h"
#include "..\miniscope\device\MiniscopeDevice.h"
#include <mmsystem.h>
#include <dsound.h>
#include "ThreadComm.h"
#include "Decimator.h"
#include <deque>
#include <vector>
#include <string>

class DirectSoundScope
{
public:
	enum { CHANNEL_CNT = 2 };
private:
    friend BOOL CALLBACK DSEnumCallback( LPGUID lpGuid, LPCSTR lpcstrDescription,  
				     LPCSTR lpcstrModule, LPVOID lpContext);
	enum { MAX_HARDWARE = 16 };
	enum { BITS_PER_SAMPLE = 16 };
	enum { BYTES_PER_SAMPLE = (BITS_PER_SAMPLE/8) };
	enum { BUFFER_COUNT = 32 };
	unsigned dev_count;
	struct dsound_dev_info
	{
		//snd_dev_info    info;
		LPGUID		    lpGuid;
		GUID		    guid;
		std::string name;
	} dev_info[MAX_HARDWARE];
	AnsiString asDeviceName;

	ThreadComm::DsoundStream stream;
	void init_waveformatex (PCMWAVEFORMAT *pcmwf, unsigned clock_rate, unsigned channel_count);
	int init_stream( struct ThreadComm::DsoundStream *strm, unsigned clock_rate, unsigned channel_count, unsigned samples_per_frame);

	bool bDeviceOpened;
	bool bRunning;
	bool bStopRequest;
	ThreadComm *threadComm;

	enum E_CONNECTION_STATE conn_state;
	unsigned char* hdr;
	unsigned int hdr_bytes;	///< skipped reply header bytes
	enum E_TRIGGER_MODE mode;
	struct S_SCOPE_SETTINGS settings, upcoming_settings;
	bool pChannelEnable[CHANNEL_CNT];
	unsigned int pSensitivity[CHANNEL_CNT];
	E_COUPLING_TYPE eCouplingType[CHANNEL_CNT];
	bool pChannelEnableUpcoming[CHANNEL_CNT];
	unsigned int pSensitivityUpcoming[CHANNEL_CNT];
	E_COUPLING_TYPE eCouplingTypeUpcoming[CHANNEL_CNT];

	bool triggered;		///< we should start sending data buffer now
	bool bManualTrigger;///< manual trigger request flag
	unsigned int lastTriggerDistance;	///< "distance" in samples from last trigger used to start capture in auto mode
										///< when there is no trigger
	unsigned int frame_position;	///< current data buffer position (0 when just sent trigger signal)

    Decimator decimator_ch1, decimator_ch2;

	std::deque<char> deque_trigger;	///< samples buffer for triggering
	short samples_ch1[384000/10];
	short samples_ch2[384000/10];
	short output_buffer_ch1[384000];
	short output_buffer_ch2[384000];
	short output_buffer_concat[384000*2];
	/** \brief Test for trigger condition
		\param len number of samples added to trigger queue recently
		\return index of first sample after triggering, < 0 if not triggered
	*/
	int TestTrigger(short* output_buffer, int len);

	typedef void (*CallbackLog)(char*);
	typedef void (*CallbackData)(short*, int);
	typedef void (*CallbackConnect)(int, char*);
	typedef void (*CallbackTrigger)(void);
	void log(char *lpData, ...);
	void SetConnectionState(enum E_CONNECTION_STATE state, const char* const descr);

    int averageA, averageB;

public:
	DirectSoundScope();
	~DirectSoundScope();
	void GetCaptureDevicesList(std::vector<std::string> &devices);
	void SetDevice(AnsiString name);
	AnsiString GetDevice(void) {
		return asDeviceName;
	}
	int Connect(void);
	int Disconnect(void);
	CallbackLog callbackLog;
	CallbackData callbackData;
	CallbackConnect callbackConnect;
	CallbackTrigger callbackTrigger;

	void DataRcvCb(short data[], unsigned int samples);
	/** \brief Request manual triggering
		\return 0 on success
	*/
	int ManualTrigger(void);
	int Run(bool state);
	int SetTriggerMode(enum E_TRIGGER_MODE);
	int SetTriggerType(int channel, enum E_TRIGGER_TYPE);
	int EnableChannel(int channel, bool state);	
	int SetCouplingType(int channel, enum E_COUPLING_TYPE);
	/** \brief Set capture buffer size
		\param size number of samples
	*/
	int SetBufferSize(int size);
	int SetSensitivity(int channel, int id);
	int SetSampling(int id);
	int SetTriggerLevel(int level);

	int GetAverageSample(int channel);
};
extern DirectSoundScope scope;


#endif
